package javax.annotation.processing;

import java.util.HashSet;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.element.Element;
import javax.lang.model.element.TypeElement;
import java.util.Objects;
import javax.lang.model.SourceVersion;
import javax.tools.Diagnostic;
import java.util.Collections;
import java.util.Set;

public abstract class AbstractProcessor implements Processor
{
    protected ProcessingEnvironment processingEnv;
    private boolean initialized;
    
    protected AbstractProcessor() {
        this.initialized = false;
    }
    
    @Override
    public Set<String> getSupportedOptions() {
        SupportedOptions supportedOptions = (SupportedOptions)this.getClass().getAnnotation(SupportedOptions.class);
        if (supportedOptions == null) {
            return Collections.emptySet();
        }
        return arrayToSet(supportedOptions.value());
    }
    
    @Override
    public Set<String> getSupportedAnnotationTypes() {
        SupportedAnnotationTypes supportedAnnotationTypes = (SupportedAnnotationTypes)this.getClass().getAnnotation(SupportedAnnotationTypes.class);
        if (supportedAnnotationTypes == null) {
            if (this.isInitialized()) {
                this.processingEnv.getMessager().printMessage(Diagnostic.Kind.WARNING, "No SupportedAnnotationTypes annotation found on " + this.getClass().getName() + ", returning an empty set.");
            }
            return Collections.emptySet();
        }
        else {
            return arrayToSet(supportedAnnotationTypes.value());
        }
    }
    
    @Override
    public SourceVersion getSupportedSourceVersion() {
        SupportedSourceVersion supportedSourceVersion = (SupportedSourceVersion)this.getClass().getAnnotation(SupportedSourceVersion.class);
        SourceVersion obj;
        if (supportedSourceVersion == null) {
            obj = SourceVersion.RELEASE_6;
            if (this.isInitialized()) {
                this.processingEnv.getMessager().printMessage(Diagnostic.Kind.WARNING, "No SupportedSourceVersion annotation found on " + this.getClass().getName() + ", returning " + obj + ".");
            }
        }
        else {
            obj = supportedSourceVersion.value();
        }
        return obj;
    }
    
    @Override
    public synchronized void init(ProcessingEnvironment processingEnvironment) {
        if (this.initialized) {
            throw new IllegalStateException("Cannot call init more than once.");
        }
        Objects.requireNonNull(processingEnvironment, "Tool provided null ProcessingEnvironment");
        this.processingEnv = processingEnvironment;
        this.initialized = true;
        return;
    }
    
    @Override
    public abstract boolean process(Set<? extends TypeElement> p0, RoundEnvironment p1);
    
    @Override
    public Iterable<? extends Completion> getCompletions(Element element, AnnotationMirror annotationMirror, ExecutableElement executableElement, String s) {
        return Collections.emptyList();
    }
    
    protected synchronized boolean isInitialized() {
        return this.initialized;
    }
    
    private static Set<String> arrayToSet(String[] array) {
        assert array != null;
        HashSet s = new HashSet(array.length);
        for (int length = array.length, i = 0; i < length; ++i) {
            s.add(array[i]);
        }
        return Collections.unmodifiableSet(s);
    }
}
